// https://github.com/ros-planning/moveit2_tutorials/blob/foxy/doc/move_group_interface/src/move_group_interface_tutorial.cpp
// maybe use this: https://github.com/ros-planning/moveit2/issues/775

#include <action_server/action_server.hpp>
#include <moveit/move_group_interface/move_group_interface.h>

ActionServer::ActionServer() : rclcpp::Node("action_server") {
    // Get the move_group we'll apply transformations to
    moveit::planning_interface::MoveGroupInterface move_group(ActionServer::_move_group_node, PLANNING_GROUP);

    // Define the planning scene
    moveit::planning_interface::PlanningSceneInterface planning_scene_interface;

    // Define a pointer to the planning scene because tutorial says it's faster
    const moveit::core::JointModelGroup* joint_model_group = move_group.getCurrentState()->getJointModelGroup(ActionServer::PLANNING_GROUP);

    // Set up the callback for settings the pose from camera/joystick/etc
    ActionServer::_goal = _move_group_node->create_subscription<geometry_msgs::msg::Pose>("/set_pose", 10, std::bind(&ActionServer::set_pose_cb, this, std::placeholders::_1));
}

void set_pose_cb(geometry_msgs::msg::Pose::SharedPtr pose) {
    move_group.setPoseTarget(pose);

    // plan the trajectory for the move
    moveit::planning_interface::MoveGroupInterface::Plan my_plan;
    bool success = (move_group.plan(my_plan) == moveit::planning_interface::MoveItErrorCode::SUCCESS);

    move_group.move();
}

int main(int argc, char **argv) {
    rclcpp::init(argc, argv);

    _move_group_node = std::make_shared<ActionServer>();
    rclcpp::spin(move_group_node);

    rclcpp::shutdown();
    return 0;
}

